#!/bin/bash
#
########################################################################
# rc.z_PSHB.sh
# pupsave-backup 1.2.1
# "Hot Backup"
# "cloned-and-hacked" into "Hot Backup" via: TwoByte, RSCR51, ZigBert 
# SaveFolderBackup via rcrsn51 2017-04-01
#
# "cloned-and-hacked" again by "gae" (GlenE77is)
# renamed to rc.z_PSHB.sh (PupSaveHotBackup) 221115-0900
# `date +%y%m%d_%H%M%S`  "ANSI" format , same as CTB, MGN, et al.
########################################################################

export gcSysDevId="PSHB"	    
export gcSysDevVer="v.1.2+"

export gcSysDevDob="221115-0900"
export gcSysDevDob="221116-0540"
export gcSysDevDob="221117-0930"
export gcSysDevDob="221118-0621"

########################################################################

# set -x

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PSHB_help_box() {
	echo "$ERRORMSG
#=======================================================================
# pupsave-backup 1.2.1
# "cloned-and-hacked" into 
# "Hot Backup" via: TwoByte, RSCR51, ZigBert 
# "SaveFolderBackup" via rcrsn51 2017-04-01
#
# "cased-and-hacked" by "gae" (GlenE77is)
# renamed to rc.z_PSHB.sh by GlenE77is 221115-0900

<b>Pupsave Backup</b> for frugal Puppy installations makes a backup copy of the live pupsave file/folder.

<b>1)</b> Running "nice -19 snapmergepuppy" 
       to block all other running programs.  
For safest results, 
       always shut down all applications and servers 
       before backing up.
     
<b>2)</b> The date 'ymd' and time 'HMS' 
       are automatically appended 
       to the backup file name.

<b>3)</b> Program backs up your system configuration 
       as it exists in the pupsave. 
<span foreground='"'red'"'>It will not backup data 
       stored outside the pupsave.</span>

<b>4)</b>To restore a backup, 
    Extract contents via Uextract Puppy Utility.

You can force the init script to 
not ignore pupsaves created by this script 
by adding a boot parameter: pfix=psavebkp

You must be running a frugal install with a pupsave 
otherwise PB will not run." > /tmp/box_help
	/usr/lib/gtkdialog/box_help "Pupsave Backup Help"
}
export -f PSHB_help_box

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PSHB_cleanup() {
	rm -rf ${PSHB_LockFile}   # /tmp/pupsave-backup-lock 
	busybox ps | grep 'PBKP_PUPSAVE_HOT_BKP' | grep -v grep | awk '{print $1}' | \
	while read pid ; do kill $pid ; done
} # end: fx(PSHB_cleanup)
#==========================================================================

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PSHB_RESTART() {
	/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "$@"
	PSHB_cleanup
    gcZeroRun="${0}"
    #-------------------------------
    . ${gcZeroRun}    &  #: $ ZERO	
    #-------------------------------
} # end: fx(PSHB_RESTART)


#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PBKP_MBrequired() { # $PBKP_PUPSAVE_PATH
    n=0
    # du : ErrFlg generates "invalid zero-length filename", due to system "end of search list". 
	a=$(du -c -m -s "${1}") #-c = --total, -m = --block-size=1m, -s = --summarize   # error flag on Zero-length file 
	read -r aa bb <<< "$a"
	echo -n "MB required: $aa  [might need even more]"
} # end: fx(PBKP_MBrequired)

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PBKP_MBavailable() { # $PBKP_BACKUP_PATH $PBKP_PUPSAVE_PATH
    n=0
	bkp="${1}"
	case "$bkp" in
		"/mnt/home"*|"/initrd/mnt/dev_save"*) drv="dev_save" ;;
		"/mnt/"*) drv="`echo "$bkp" | cut -f 3 -d '/'`";;
		*) PSHB_RESTART "You can only back up a pupsave to a device mounted on /mnt ..." ; return 1 ;;
	esac
	a="`df -k | grep "$drv" | awk '{ print $4 }'`"
	if [ "$a" = "" ] ; then
		a=0
	else
		a=$((a/1024)) #mb
	fi
	[ "$drv" = "dev_save" ] && drv="/initrd/mnt/dev_save"
	lbldrv="/mnt/$drv"
	[ "$drv" = "/initrd/mnt/dev_save" ] && lbldrv=$drv
	#
	fs=$(grep "${drv} " /proc/mounts | cut -f 3 -d ' ')
	if [ "$fs" = "" ] ; then
		PSHB_RESTART "$drv: STORAGE DEVICE NOT MOUNTED... You can only back up a pupsave to a device mounted on /mnt"
		return 1
	fi
	if [ "$SAVEFOLDER" ] ; then
		case $fs in ext[2-4]) n=1 ;;
			*) PSHB_RESTART "$drv has a invalid filesystem: $fs ... You can only back up savefolders to a ext2/ext3/ext4 partition..." ; return 1 ;;
		esac
	fi
	aa="$(du -c -m -s "${2}")"   #-c = --total, -m = --block-size=1m, -s = --summarize   # error flag on Zero-length file 
	read -r b ccc <<< "$aa"
    #
	echo -n "MB available on ${lbldrv}: $a"
	if [ $((a-b)) -le 100 ]; then # not enough space
		/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "NOT ENOUGH FREE DISK SPACE on $drv..."
		return 1
	fi
}

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PBKP_Bkpfilename() { # $PBKP_PUPSAVE_PATH echo into Gui
	echo -n "Backup file name: ${PBKP_PUPSAVE_NAME}.BKP-`date +%y%m%d-%H%M-%S`"
} # end: fx(PBKP_Bkpfilename)

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PBKP_validate_BACKUP_location() { # $PBKP_BACKUP_PATH echo into Gui 
	if [ "${1}" != "" ]; then 
		IFS="/" read -r a direntry level2 level3 level4 <<< "${1}"
		if [ "$direntry" = "mnt" ] && [ "$level2" != "" ] ; then
			echo -n "${1}"
		elif [ "$direntry"/"$level2" = "initrd/mnt" ] && [ "$level3" != "" ] ; then
			echo -n "${1}"
		fi
    else
		echo -n "$PBKP_LOCDEFAULT"
	fi
} # end: fx(PBKP_validate_BACKUP_location)
 
#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function compression_dialog() {   # Not Called 
    n=0
			#~ TAROPT="-z"
			#~ TAREXT='tar.gz'
	#~ /usr/lib/gtkdialog/box_yesno "Pupsave Backup-Compress" "Do you want to compress the backup ${destfile} ?
    #~ May take a bit longer but save some space."
	#~ if [ $? -eq 0 ] ; then
		#~ /usr/lib/gtkdialog/box_yesno --yes-label "gzip" --no-label "xz" --yes-icon "gtk-execute" --no-icon "gtk-execute" \
		#~ "Pupsave Backup-Compress" "Select compression type: 'gzip' or 'xz'"
		#~ if [ $? -eq 0 ] ; then
			#~ TAROPT="-z"
			#~ TAREXT='tar.gz'
		#~ else
			#~ TAROPT="-J"
			#~ TAREXT='tar.xz'
		#~ fi
	#~ fi
} # end: fx(compression_dialog)   # Not Called


#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function Show_SplashIt() {   # Not Called 
    n=0
/usr/lib/gtkdialog/box_splash -fg "${2}" -bg "${3}"  -timeout "${4}" -text "${1}" &
	GtkPid_SplashIt=$!	# capture PID
	kill ${GtkPid_SplashIt} 
} # end: fx(Show_SplashIt)


#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function show_splash() {   # Not Called
	/usr/lib/gtkdialog/box_splash -close never  -text "$@" &
	pidx=$!
    # kill ${pidx}  # later
} # end: fx(show_splash) # Not Called

#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function PBKP_Backup() { # $PBKP_PUPSAVE_PATH $PBKP_BACKUP_PATH
    n=0
    /usr/lib/gtkdialog/box_splash -fg yellow -bg green -close never -text "... Running ${0} ..." &
    RETVAL=$?
	GtkPid_Splash_Running=$!	# capture PID
	sleep 1
    destfile=".${2}"/"`expr "${1}" | awk -F / '{print $NF}'`".BKP_`date +%y%m%d_%H%M%S`

                #~ if [ -e "${destfile}" -o -e "${destfile}.tar.gz" -o -e "${destfile}.tar.xz" ]; then
                    #~ /usr/lib/gtkdialog/box_ok "Pupsave Backup" error "This Pupsave file already exists.  
                    #~ Wait one minute and the file name will change."
                    #~ return 1
                #~ fi
                    #~ kill ${pidx}

    # working gae cull extra option selection
	if [ $PUPMODE -eq 13 ] ; then #flash

                    #~ /usr/lib/gtkdialog/box_yesno --ok-cancel "Pupsave Backup" "Running in PUPMODE 13 - flash media
                    #~ Click 'OK' to save session to save${type} and proceed with the backup...
                    #~ Backup file name: ${destfile}"
                    #~ if [ $? -eq 0 ] ; then
                        #~ show_splash "{Saving Session} to save ${type} ..." &
                        #~ sleep 3
                        #~ kill $pidx
                        #~ snapmergepuppy
                    #~ else
                        #~ return 1
                    #~ fi
        #
    	nice -19 snapmergepuppy
        #
	fi # end: if [ $PUPMODE -eq 13 ]

	CWDIR=$PWD

	kill ${GtkPid_Splash_Running} # after return # kill splash green # before TarOpt
    sleep .5
   
	#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) test if [ ${SF} is realvalue ] "  

	if [ "$SAVEFOLDER" ] ; then   # val = "1"
		# compression_dialog # set TAROPT TAREXT
		TAROPT="-z"
		TAREXT='tar.gz'
		[ "$TAROPT" ] && destfile=${destfile}.${TAREXT}

        # working gae splash
        /usr/lib/gtkdialog/box_splash -fg blue -bg orange -close never -text "(1) Creating ${destfile##*/} ... Please Wait for TarOpt to run ..." &
        RETVAL=$?
        GtkPid_Splash_TarCreate=$!	# capture PID
        #~ sleep 3
        #~ kill ${GtkPid_Splash} # after return

       
		#~ if [ "$TAROPT" ] ; then   # if true [ $TAROPT -eq somevalue ]

			cd ${1%/*} # dirname $PBKP_PUPSAVE_PATH
			#basename destifile / basename $PBKP_PUPSAVE_PATH
			echo ; echo "tar -c ${TAROPT} -f ${destfile##*/} ${1##*/}/" #debug
			tar -c ${TAROPT} -f .${destfile##*/} ${1##*/}/
			RETVAL=$?
			cd "$CWDIR"
			sync

                    #~ else    # not zip # this block CANNOT run. 
                        #~ show_splash "(1+) else [ "TAROPT" <> "zip" ]" # OK 
                        #~ mkdir -p ${destfile}
                        #~ if which rsync ; then
                            #~ rsync -aX ${1}/ ${destfile}/
                            #~ RETVAL=$?
                        #~ else
                            #~ find "${1}" -mindepth 1 -maxdepth 1 | while read i ; do
                                #~ cp -a --remove-destination ${i} ${destfile}/
                                #~ RETVAL=$?
                                #~ sync
                            #~ done
                        #~ fi
                        #~ sync

                        #~ kill ${pidx}

		#~ fi # end: if [ "$TAROPT" ]

        sleep 3
        kill ${GtkPid_Splash_TarCreate}

	else #saveFile # Not SaveFolder # val = "0"

	    #~ show_splash "(2) else [ savefile ]"
        #~ sleep 3
		#~ kill $pidx

        # Else Not SaveFolder
		#~ compression_dialog #set TAROPT TAREXT
        TAROPT="-z"
        TAREXT='tar.gz'
        # Else Not SaveFolder
        sleep .5

		/usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "(1) copy to ${destfile##*/} "  
        
        # Else Not SaveFolder
		cp -f -v "${1}" "${destfile}" 2> /dev/null
		RETVAL=$?
		sync
        sleep 1
		/usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "(1) e2fsck  correct format  ${destfile##*/} "  

        # Else Not SaveFolder, so copy, correct format, to destfile, for TarOpt
		e2fsck -y "${destfile}" 2> /dev/null

        # gcRETVAL=0 # after "fi"

	    # Else Not SaveFolder
    	#~ if [ "$TAROPT" ] ; then     # Have this !  No Test Req. 
			prevdfile=${destfile}
			destfile=${destfile}.${TAREXT}

           # Else Not SaveFolder
		#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) TarOpt into proper ${destfile##*/} "  
 
 			cd ${1%/*} # dirname $PBKP_PUPSAVE_PATH
			tar -c ${TAROPT} -f ${destfile##*/} ${prevdfile##*/} #basename $destfile
			# RETVAL=$? # after "fi"
			rm -f $prevdfile

			sleep 3
        	kill ${GtkPid_Splash_TarCreate} # after return

		#~ fi # end: if [ "$TAROPT" ]       # Have this !  No Test Req. 
	fi # end: if [ "$SAVEFOLDER" ]   else #saveFile 

    gcRETVAL="${RETVAL}"

            # mv /etc/xloadedx /root/.XLOADED      # Distractor
            # sync

    #-----------------------------------------------------------------------
    if [ $gcRETVAL -eq 0 ] ; then   # Backup Completed OK
        /usr/lib/gtkdialog/box_splash -fg yellow -bg red -close never -text "(2) DONE: Save${gcPshbType} Backup : ${destfile}             "&
        RETVAL=$?
        GtkPid_Splash_Done=$!	# capture PID
        sleep 5
        kill ${GtkPid_Splash_Done}
	else # NOT	if [ $gcRETVAL -eq 0 ]
		rm -rf $destfile
		/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "Error(s) creating $destfile" 
		return 1
	fi # end: if [ $gcRETVAL -eq 0 ]

} # end: fx(PBKP_Backup)





#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
####################### Execution #######################################
#######################   main()  #######################################
#######################   main()  #######################################
#######################   main()  #######################################

#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( main )"  

#--- main ------------------------------
if [ -f /tmp/pupsave-backup-lock ];then
	/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "Pupsave Backup is already running."
	exit 1
fi

export PSHB_LockFile="/tmp/pupsave-backup-lock"
touch ${PSHB_LockFile}   # /tmp/pupsave-backup-lock

#--- main ------------------------------
export -f PSHB_cleanup
trap PSHB_cleanup SIGHUP SIGINT SIGTERM

#--- main ------------------------------
# Verify frugal type installation.
. /etc/rc.d/PUPSTATE
export PUPMODE
case $PUPMODE in
	12|13) ok=1 ;;
	*) ERRORMSG="Hot Backup can only be used if there is an active pupsave..." ;;
esac

#--- main ------------------------------
if [ "$ERRORMSG" ] ; then
	PSHB_cleanup
	echo "$ERRORMSG"
	ERRORMSG="<b><span foreground='"'red'"'>${ERRORMSG}</span></b>"
	[ $DISPLAY ] && PSHB_help_box
	PSHB_cleanup
	exit 1
fi

#--- main ------------------------------
#Find and verify path of currently mounted save file.
SF="`echo $PUPSAVE | cut -f 3 -d ","`" # /savefilename or /xxx/savefilename

#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) test if [ ! "$SF" ]  "  
if [ ! "$SF" ] ; then
	/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "Currently using a full partition..."
	PSHB_cleanup
	exit
fi

#--- main ------------------------------
#SF=${SF#/}
#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) test if [ ! -e "/mnt/home${SF}" ] "  
if [ ! -e "/mnt/home${SF}" ]; then
	/usr/lib/gtkdialog/box_ok "Pupsave Backup" error "'/mnt/home${SF}' does not exist!"
	PSHB_cleanup
	exit 1
fi
if [ -d "/mnt/home/${SF}" ]; then  # else {SF} is DIRECTORY 
#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) elif [ -d "/mnt/home/${SF}" ]"  
	export SAVEFOLDER=1
	export REFRESH_INTERVAL=1000 #1 seconds
	export gcPshbType=folder
else #  {SF} is FOLDER
#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) else  {SF} is FOLDER "  
	export SAVEFOLDER=0 # empty value   # working gae # orig error here !
	export REFRESH_INTERVAL=1000 #1 seconds
	export gcPshbType=file
fi

#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup"  complete "( ) ---main--- export variables "  

#--- main ------------------------------
# Export variables
export PBKP_PUPSAVE_PATH="/mnt/home${SF}"
export PBKP_PUPSAVE_NAME="${SF##*/}" #${SF##*/} = basename $SF
export PBKP_LOCDEFAULT="/mnt/home${PSUBDIR}"
export PBKP_BACKUP_PATH=$(PBKP_validate_BACKUP_location)

#--- main ------------------------------
# building date string with "%S" seconds, and gui interval of 1 second, so no possible dupe. 
#~ case "$PBKP_PUPSAVE_NAME" in *".BKP-"*) # useing date with "%S" seconds, and interval of 1 second, so no possible dupe. 
	#~ /usr/lib/gtkdialog/box_ok "Pupsave Backup" warning "
    #~ '${PBKP_PUPSAVE_NAME}' seems to be a file produced by this script.
     #~ It's not advised to use it, the file name might become too large..." 
#~ esac

#--- main ------------------------------
# Exported functions
export -f compression_dialog PSHB_RESTART PBKP_MBrequired PBKP_MBavailable PBKP_Bkpfilename PBKP_Backup PBKP_validate_BACKUP_location

#--- main ------------------------------
# For fileselect dialog default directory
cd /mnt

#--- Main GUI ------------------------------
# Some escapement is necessary when this sed is used to allow comments in gui.
export PBKP_PUPSAVE_HOT_BKP='
<window title="rc.z_PSHB.sh cfg PupSave :'${gcSysDevDob}':" icon-name="gtk-convert" default_width="750">
   <vbox>
      <frame>
         <text editable="false" use-markup="true" xalign="0.001">
            <label>"<span color='"'blue'"'>Pupsave '${gcPshbType}' to backup</span>"</label>
         </text>
         <hbox>
            <entry editable="false">
               <variable>PBKP_PUPSAVE_PATH</variable>
               <sensitive>"false"</sensitive>
               <input>echo -n "'\$PBKP_PUPSAVE_PATH'"</input>
            </entry>
         </hbox>
         <text editable="false" use-markup="true" xalign="0.001">
            <label>"<span color='"'blue'"'>Save backup to /mnt/??</span>"</label>
         </text>
         <hbox>
            <entry editable="false" accept="directory">
               <variable>PBKP_BACKUP_PATH</variable>
               <input>PBKP_validate_BACKUP_location "'\$PBKP_BACKUP_PATH'"</input>
            </entry>
            <button>
               <input file stock="gtk-open"></input>
               <action type="fileselect">'PBKP_BACKUP_PATH'</action>
               <action>refresh:TEXT2</action>
            </button>
         </hbox>
      </frame>
      <frame Information>
         <text height-request="" xalign="0">
            <variable>TEXT1</variable>
            <input>PBKP_MBrequired "'\$PBKP_PUPSAVE_PATH'"</input>
         </text>
         <text height-request="20" xalign="0">
            <variable>TEXT2</variable>
            <input>PBKP_MBavailable "'\$PBKP_BACKUP_PATH'" "'\$PBKP_PUPSAVE_PATH'"</input>
         </text>
         <text height-request="20" xalign="0">
            <variable>BKPFILE</variable>
            <input>PBKP_Bkpfilename "'\$PBKP_PUPSAVE_PATH'"</input>
         </text>
         <timer milliseconds="true" interval="'${REFRESH_INTERVAL}'" visible="false"> 
           <action>refresh:TEXT1</action>
           <action>refresh:TEXT2</action>
           <action>refresh:BKPFILE</action>
           <action>refresh:PBKP_BACKUP_PATH</action>
         </timer>
      </frame>
      <hbox>
         <button help><action>PSHB_help_box &</action></button>
         <button>
            <input file icon="gtk-save"></input>
            <label>Backup</label>
            <action>EXIT:BACKUP</action>
            <action>PBKP_Backup "'\$PBKP_PUPSAVE_PATH'" "'\$PBKP_BACKUP_PATH'" &</action>
         </button>
         <button use-stock="true" label="gtk-quit" has-focus="true">
            <action>EXIT:Exit</action>
         </button>
      </hbox>
   </vbox>
</window>
'
eval "`gtkdialog --center -p PBKP_PUPSAVE_HOT_BKP`"


#---------------------------------
if [ "$EXIT" = "BACKUP" ] ; then
	if PBKP_MBavailable "$PBKP_BACKUP_PATH" ; then
		if PBKP_Backup "$PBKP_PUPSAVE_PATH" "$PBKP_BACKUP_PATH" ; then
			PSHB_cleanup
			exit 0 #success
		fi
	fi
	#error
	PSHB_cleanup
	cd -
	$0 & #reenter
	exit 1
fi

PSHB_cleanup

### END ###

    # working gae must clear lingering gtkdialog pid 

#########################################################################
